package jmslexamples.simple;

import java.util.Vector;

import com.softsynth.jmsl.*;
import com.softsynth.jmsl.jsyn2.JSynMusicDevice;
import com.softsynth.jmsl.jsyn2.JSynUnitVoiceInstrument;
import com.softsynth.jmsl.score.*;
import com.softsynth.jmsl.score.transcribe.*;
import com.softsynth.jmsl.util.TimeSignature;

/**
 * 
 * Transcribe two MusicShapes and generate a Score.
 * 
 * Dim 0 of MusicShapes must be absolute time. Use musicShape.integrate(0) to convert durations to
 * abs time
 * 
 * Melody 2 uses quarter tones
 * 
 * @author Nick Didkovsky, (c) 2002 Nick Didkovsky, All Rights reserved.
 */

public class TranscribeAndNotate {
    MusicShape melody1;
    MusicShape melody2;
    Score score;
    Orchestra orchestra;

    public TranscribeAndNotate() {

    }

    private void initMusicDevices() {
        JMSL.clock = new DefaultMusicClock();
        MusicDevice dev = JSynMusicDevice.instance();
//        dev.edit(new java.awt.Frame());
        dev.open();
    }

    private void buildOrchestra() {
        orchestra = new Orchestra();
        // use 8 voice polyphonic stereo JSyn instrument
        Instrument ins1 = new JSynUnitVoiceInstrument(8,
                com.softsynth.jmsl.jsyn2.unitvoices.AutoPanningSawtoothBL.class.getName());
        orchestra.addInstrument(ins1, "Stereo Autopanning JSyn Instrument");

        Instrument ins2 = new JSynUnitVoiceInstrument(8, com.softsynth.jmsl.jsyn2.unitvoices.FMVoice.class.getName(),
                com.softsynth.jmsl.jsyn2.unitvoices.FMVoice.PRESET_WOODDRUM);
        orchestra.addInstrument(ins2, "wood");

        score.setOrchestra(orchestra);

        if (ins1.getNumOutputs() == 2) {
            // pan L/R
            score.getOrchestra().getJMSLMixerContainer().panAmpChange(0, 0, 0.5);
            score.getOrchestra().getJMSLMixerContainer().panAmpChange(1, 1, 0.5);
        }

    }

    void buildScore() {
        // Prepare a Score, 2 staves, width, height
        score = new Score(2, 1024, 800);
        score.addMeasure();
        score.getMeasure(0).getStaff(1).setClef(Clef.BASS_CLEF);
    }

    void buildMusicShapes() {
        // create musicshape data algorithmically
        melody1 = new MusicShape(4);
        for (int i = 0; i < 6; i++) {
            double duration = JMSLRandom.choose(3, 6);
            double pitch = JMSLRandom.choose(48, 60);
            double amp = JMSLRandom.choose(0.2, 0.7);
            double hold = duration;
            melody1.add(duration, pitch, amp, hold);
        }
        melody2 = new MusicShape(4);
        for (int i = 0; i < 17; i++) {
            double duration = JMSLRandom.choose(2.0);
            double pitch = JMSLRandom.choose(48, 60);
            // let's add a qtr tone roughly half the time
            if (JMSLRandom.choose() < 0.5)
                pitch += 0.5;
            double amp = JMSLRandom.choose(0.2, 0.7);
            double hold = duration * 0.8;
            melody2.add(duration, pitch, amp, hold);
        }
    }

    void transcribe() throws SearchPathListExpansionException, ElementMissedException {
        // set up allowed subdivisions
        BeatDivisionSchemeList.defaultSetup();
        Transcriber transcriber = new Transcriber();
        transcriber.setScore(score);

        TimeSignature ts = new TimeSignature(3, 4);
        Vector tsVector = new Vector();
        tsVector.addElement(ts);
        tsVector.addElement(new TimeSignature(5, 4));
        transcriber.setTimeSignatures(tsVector);
        transcriber.setLoopTimeSignatures(true);

        // Set up time signatures. Only need to give it one 4/4 bar and
        // transcriber will continue
        // with 4/4 throughout
        // NEW: The following code is done by default in Transcriber()
        // constructor.
        // Uncomment and change if you want 5/4 for example, or a sequence of
        // time sigs
        // TimeSignature ts = new TimeSignature(4, 4);
        // Vector tsVector = new Vector();
        // tsVector.addElement(ts);
        // transcriber.setTimeSignatures(tsVector);

        // transcribe first melody
        score.setCurrentStaffNumber(0);
        score.rewind();
        transcriber.setSourceMusicShape(melody1);
        melody1.integrate(0);
        transcriber.transcribe();

        // transcribe second melody
        score.setCurrentStaffNumber(1);
        score.rewind();
        transcriber.setSourceMusicShape(melody2);
        melody2.integrate(0);
        transcriber.transcribe();

        // here's a nice trick: transcribe first melody an octave higher and stick it on the second
        // track of the top staff
        score.setCurrentStaffNumber(0);
        score.setCurrentTrackNumber(1); // tracks are 0 and 1
        score.rewind();
        melody1.transpose(12, 0, melody1.size() - 1, 1);
        transcriber.setSourceMusicShape(melody1);
        // notice we do not integrate again
        transcriber.transcribe();
    }

    /**
     * @return Returns the score.
     * @throws ElementMissedException
     * @throws SearchPathListExpansionException
     */
    public Score generateScore() throws SearchPathListExpansionException, ElementMissedException {
        buildScore();
        buildOrchestra();
        buildMusicShapes();
        transcribe();
        return score;
    }

    public static void main(String args[]) {
        JMSLRandom.randomize();
        JMSL.clock.setAdvance(0.3);
        TranscribeAndNotate transcribeAndNotate = new TranscribeAndNotate();
        transcribeAndNotate.initMusicDevices();
        
        Score score = null;
        try {
            score = transcribeAndNotate.generateScore();
        } catch (SearchPathListExpansionException e1) {
            e1.printStackTrace();
        } catch (ElementMissedException e1) {
            e1.printStackTrace();
        }
        final ScoreFrame myScoreFrame = new ScoreFrame();
        myScoreFrame.addScore(score);
        myScoreFrame.addWindowListener(new java.awt.event.WindowAdapter() {
            public void windowClosing(java.awt.event.WindowEvent e) {
                myScoreFrame.quit();
            }
        });
        myScoreFrame.setVisible(true);
    }

}
